@@ -87,7 +87,7 @@ typedef enum {
 #define OUTPUT_FRAGMENT 9
 #define OUTPUT_ZONEID   10
 
-#define NUM_COMPONENTS 11 /* including "url" */
+#define NUM_COMPONENTS 10 /* excluding "url" */
 
 #define PROGNAME        "trurl"
 
@@ -204,7 +204,8 @@ static void help(void)
     "      --quiet                      - Suppress (some) notes and comments\n"
     " URL COMPONENTS:\n"
     "  ", stdout);
-  for(i = 0; i< NUM_COMPONENTS; i++) {
+  fputs("url, ", stdout);
+  for(i = 0; i< NUM_COMPONENTS ; i++) {
     printf("%s%s", i?", ":"", variables[i].name);
   }
   fputs("\n", stdout);
@@ -316,6 +317,7 @@ static void trurl_warnf(struct option *o, const char *fmt, ...)
 struct string qpairs[MAX_QPAIRS]; /* encoded */
 struct string qpairsdec[MAX_QPAIRS]; /* decoded */
 int nqpairs; /* how many is stored */
+bool query_is_modified = false;
 
 static void trurl_cleanup_options(struct option *o)
 {
@@ -820,6 +822,9 @@ static void get(struct option *o, CURLU *uh)
               break;
             }
           }
+          else
+            errorf(o, ERROR_GET, "\"%.*s\" is not a recognized URL component",
+                   (int)vlen, ptr);
         }
         ptr = end + 1; /* pass the end */
       }
@@ -876,6 +881,9 @@ static const struct var *setone(CURLU *uh, const char *setline,
     v = comp2var(setline, vlen);
     if(v) {
       CURLUcode rc;
+      if((v->part == CURLUPART_HOST) && ('[' == ptr[1]))
+        /* when setting an IPv6 numerical address, disable URL encoding */
+        urlencode = false;
       rc = curl_url_set(uh, v->part, ptr[1] ? &ptr[1] : NULL,
                         (o->curl ? 0 : CURLU_NON_SUPPORT_SCHEME)|
                         (urlencode ? CURLU_URLENCODE : 0) );
@@ -1077,6 +1085,7 @@ static void trim(struct option *o)
       }
       free(temp);
     }
+    query_is_modified = true;
   }
 }
 
@@ -1214,21 +1223,24 @@ static void extractqpairs(CURLU *uh, struct option *o)
 
 static void qpair2query(CURLU *uh, struct option *o)
 {
-  int i;
-  char *nq = NULL;
-  for(i = 0; i<nqpairs; i++) {
-    char *oldnq = nq;
-    nq = curl_maprintf("%s%s%s", nq?nq:"",
-                       (nq && *nq && *(qpairs[i].str))? o->qsep: "",
-                       qpairs[i].str);
-    curl_free(oldnq);
+  if(query_is_modified) {
+    int i;
+    char *nq = NULL;
+    for(i = 0; i<nqpairs; i++) {
+      char *oldnq = nq;
+      nq = curl_maprintf("%s%s%s", nq?nq:"",
+                         (nq && *nq && *(qpairs[i].str))? o->qsep: "",
+                         qpairs[i].str);
+      curl_free(oldnq);
+    }
+    if(nq) {
+      int rc = curl_url_set(uh, CURLUPART_QUERY, nq, 0);
+      if(rc)
+        trurl_warnf(o,
+                    "internal problem: failed to store updated query in URL");
+    }
+    curl_free(nq);
   }
-  if(nq) {
-    int rc = curl_url_set(uh, CURLUPART_QUERY, nq, 0);
-    if(rc)
-      trurl_warnf(o, "internal problem");
-  }
-  curl_free(nq);
 }
 
 /* sort case insensitively */
@@ -1254,6 +1266,7 @@ static void sortquery(struct option *o)
     /* not these two lists may no longer be the same order after the sort */
     qsort(&qpairs[0], nqpairs, sizeof(struct string), cmpfunc);
     qsort(&qpairsdec[0], nqpairs, sizeof(struct string), cmpfunc);
+    query_is_modified = true;
   }
 }
 
@@ -1282,7 +1295,7 @@ static void replace(struct option *o)
       if(strncmp(q, key.str, key.len))
         continue;
       free(qpairs[i].str);
-      curl_free(qpairsdec[i].str);
+      free(qpairsdec[i].str);
       /* this is a duplicate remove it. */
       if(replaced) {
         qpairs[i].len = 0;
@@ -1309,6 +1322,7 @@ static void replace(struct option *o)
                   key.str);
       addqpair(key.str, strlen(key.str), o->jsonout);
     }
+    query_is_modified = true;
   }
 }
 static CURLUcode seturl(struct option *o, CURLU *uh, const char *url)
@@ -1466,6 +1480,7 @@ static void singleurl(struct option *o,
     /* append query segments */
     for(p = o->append_query; p; p = p->next) {
       addqpair(p->data, strlen(p->data), o->jsonout);
+      query_is_modified = true;
     }
 
     sortquery(o);
